// Game script that provides a HUD (with stamina bar) and handles a sprint key.

// ---------------------------------------------------------------------------------------------------------------
// HUD constants

// 32-bit ARGB modulation color (in hex) for base/frame elements, alpha = 102, RGB = 255 255 255
const c_u4HudBaseARGB = 0x66FFFFFF;

// health bar color and alpha
const c_u4HHealthBarARGB = 0xCCD4BE20;
// stamina bar color and alpha
const c_u4HStamBarARGB = 0xCC009BF4;

// start flashing the health icon if health is below this value
const c_fFlashHealthBelow = 30.0;

// pos and dimensions of the bars frame in bars.tga
const c_iBarsFrame_SrcX = 5;
const c_iBarsFrame_SrcY = 4;
const c_iBarsFrame_SrcW = 246;
const c_iBarsFrame_SrcH = 33;
// pos and dims of the filled bar graphic in bars.tga
const c_iBar_SrcX = 37;
const c_iBar_SrcY = 49;
const c_iBar_SrcW = 207;
const c_iBar_SrcH = 8;
// pos, dimensions and offset inside frame of the stamina symbol in bars.tga (used to make it pulsate when sprint key is pressed)
const c_iStamIcon_SrcX = 14;
const c_iStamIcon_SrcY = 21;
const c_iStamIcon_SrcW = 11;
const c_iStamIcon_SrcH = 10;
const c_iStamIcon_OffsetX = 8;
const c_iStamIcon_OffsetY = 16;
// pos, dimensions and offset inside frame of the health symbol in bars.tga (used to make it pulsate when low health)
const c_iHealthIcon_SrcX = 18;
const c_iHealthIcon_SrcY = 7;
const c_iHealthIcon_SrcW = 11;
const c_iHealthIcon_SrcH = 12;
const c_iHealthIcon_OffsetX = 12;
const c_iHealthIcon_OffsetY = 2;

// offset for filled health bar inside bars frame
const c_iHealthBar_OffsetX = 32;
const c_iHealthBar_OffsetY = 5;
// offset for filled stamina bar inside bars frame
const c_iStamBar_OffsetX = 28;
const c_iStamBar_OffsetY = 19;

// pos and dimensions of the frame in ammo.tga
const c_iAmmoFrame_SrcX = 5;
const c_iAmmoFrame_SrcY = 4;
const c_iAmmoFrame_SrcW = 54;
const c_iAmmoFrame_SrcH = 33;

// character dimension in digits.tga
const c_iDigitW = 24;
const c_iDigitH = 27;
// character width during output (slightly horizontally compressed to fit in frame)
const c_iDigitOutW = 19;
const c_iDigitSpacing = 15;

// ---------------------------------------------------------------------------------------------------------------
// Game constants

// sprint key uses the USER1 general purpose key bind
uCMD_SPRINT <- uCMD_USER1;


// ---------------------------------------------------------------------------------------------------------------
// helper class that handles HUD drawing for HW rendering (OnPaintHW3D)
class HUD_TI
{
	texHealth = null;
	texAmmo = null;
	texDigits = null;


	constructor()
	{
		// init required resources
		texHealth = TP_Engine.GetTexture("scripts/bars.tga");
		texAmmo = TP_Engine.GetTexture("scripts/ammo.tga");
		texDigits = TP_Engine.GetTexture("scripts/digits.tga");
	}

	// draw a number in the range 0 to 999
	function DrawNumber(iNumber, xpos, ypos, bRightAlign = false)
	{
		// output width and height for one digit
		local dw = c_iDigitOutW;
		local dh = c_iDigitH;

		if (iNumber > 999)
			iNumber = 999;
		else if (iNumber < 0)
			iNumber = 0;

		local s = iNumber.tostring();
		if (bRightAlign)
			xpos -= c_iDigitSpacing * s.len();

		foreach (c in s)
		{
			local i = c - '0';
			TP_Engine.DrawTexture(texDigits,  i * c_iDigitW, 0, c_iDigitW, c_iDigitH,  xpos, ypos, dw, dh,  c_u4HudBaseARGB, 1);
			xpos += c_iDigitSpacing;
		}
	}

	function Draw()
	{
		local plr = TP_Engine.GetPlayer();
		if (!plr)
			return;

		local sw = TP_Engine.GetScreenWidth();
		local sh = TP_Engine.GetScreenHeight();

		const xmargin = 30;
		const ymargin = 30;

		// --------------
		// health display
		// --------------
		if (texHealth)
		{
			// top-left corner of bars frame on screen
			local x = xmargin;
			local y = sh - c_iBarsFrame_SrcH - ymargin;

			// draw frame
			TP_Engine.DrawTexture(
				texHealth,
				c_iBarsFrame_SrcX, c_iBarsFrame_SrcY, c_iBarsFrame_SrcW, c_iBarsFrame_SrcH,
				x, y, c_iBarsFrame_SrcW, c_iBarsFrame_SrcH,
				c_u4HudBaseARGB, 1);

			// draw health bar

			// bar width based on health amount
			local health = plr.GetHealth();
			local w = (c_iBar_SrcW * (health / 100.0)).tointeger();

			TP_Engine.DrawTexture(
				texHealth,
				c_iBar_SrcX, c_iBar_SrcY, w, c_iBar_SrcH,
				x+c_iHealthBar_OffsetX, y+c_iHealthBar_OffsetY, w, c_iBar_SrcH,
				c_u4HHealthBarARGB, 1);

			// draw stamina bar

			// get stamina level from player (requires the player to use the custom CPlayerTI script class with stamina support)
			local stamina = 100.0;
			if (plr instanceof CPlayerTI)
				stamina = plr.GetStamina();
			w = (c_iBar_SrcW * (stamina / 100.0)).tointeger();

			TP_Engine.DrawTexture(
				texHealth,
				c_iBar_SrcX, c_iBar_SrcY, w, c_iBar_SrcH,
				x+c_iStamBar_OffsetX, y+c_iStamBar_OffsetY, w, c_iBar_SrcH,
				c_u4HStamBarARGB, 1);

			// flash stamina icon while sprint key is pressed with no stamina left
			if (plr instanceof CPlayerTI && plr.bSprinting && stamina == 0.0)
			{
				// to flash the icon (with a glow) just take a sine based on sim time to calculate an alpha value
				local fFlashFactor = sin(TP_Engine.GetElapsedSimTime() * 6.28) * 0.5 + 0.5;
				local u4FlashColor = TP_Engine.rgba(255, 255, 255, (255.0 * fFlashFactor).tointeger());

				// glow is drawn with additive/burn blend mode
				TP_Engine.DrawTexture(
					texHealth,
					c_iStamIcon_SrcX, c_iStamIcon_SrcY, c_iStamIcon_SrcW, c_iStamIcon_SrcH,
					x+c_iStamIcon_OffsetX, y+c_iStamIcon_OffsetY, c_iStamIcon_SrcW, c_iStamIcon_SrcH,
					u4FlashColor, 2);
			}

			// flash health icon when health is low
			if (health < c_fFlashHealthBelow)
			{
				// to flash the icon (with a glow) just take a sine based on sim time to calculate an alpha value
				// but increase the flash frequency as health lowers
				local urgency = (c_fFlashHealthBelow - health) / c_fFlashHealthBelow;
				local fFlashFactor = sin(TP_Engine.GetElapsedSimTime() * 6.28 * (1.0 + 4.0 * urgency)) * 0.5 + 0.5;
				if ( plr.bDead() )
					// flat-lined
					fFlashFactor = 0.0;
				local u4FlashColor = TP_Engine.rgba(255, 255, 255, (255.0 * fFlashFactor).tointeger());

				// glow is drawn with additive/burn blend mode
				TP_Engine.DrawTexture(
					texHealth,
					c_iHealthIcon_SrcX, c_iHealthIcon_SrcY, c_iHealthIcon_SrcW, c_iHealthIcon_SrcH,
					x+c_iHealthIcon_OffsetX, y+c_iHealthIcon_OffsetY, c_iHealthIcon_SrcW, c_iHealthIcon_SrcH,
					u4FlashColor, 2);
			}
		}

		// --------------
		// ammo display
		// --------------
		if (texAmmo && texDigits)
		{
			// only draw this if player is holding a gun (with a valid max ammo count)
			local obj = plr.pinsHeldObject();
			if (obj instanceof CGun && obj.iMaxAmmo > 0)
			{
				// top-left corner of ammo frame on screen
				local x = xmargin + c_iBarsFrame_SrcW + 4;
				local y = sh - c_iBarsFrame_SrcH - ymargin;

				// draw frame
				TP_Engine.DrawTexture(
					texAmmo,
					c_iAmmoFrame_SrcX, c_iAmmoFrame_SrcY, c_iAmmoFrame_SrcW, c_iAmmoFrame_SrcH,
					x, y, c_iAmmoFrame_SrcW, c_iAmmoFrame_SrcH,
					c_u4HudBaseARGB, 1);

				// top-right corner for right-aligned ammo count text
				local tx = c_iAmmoFrame_SrcW - 8;
				local ty = (c_iAmmoFrame_SrcH - c_iDigitH) / 2;

				// draw ammo count (right-aligned)
				DrawNumber(obj.iAmmo, x+tx, y+ty, true);
			}
		}
	}
}


// ---------------------------------------------------------------------------------------------------------------
// A game script that provides custom HUD and other extended game mechanics
class GameTI extends GameScript
{
	hud = null;


	function OnScriptInit(bSaveGame)
	{
		// no HUD provided if game is running with software rendering (too limiting or requires uglier fallback)
		if ( !TP_Engine.IsSoftwareMode() )
			hud = HUD_TI();
	}

	function OnPaintHW3D()
	{
		if (hud)
			hud.Draw();
	}

	function OnHandleInput()
	{
		// handle sprint key (requires player to use the custom CPlayerTI script class)
		local plr = TP_Engine.GetPlayer();
		if (plr && plr instanceof CPlayerTI)
		{
			plr.UpdateSprintState(!plr.bDead() && bKeyPressed(uCMD_SPRINT));
		}
	}
}
