// Player script that provides stamina/sprint functionality to the player.

// ---------------------------------------------------------------------------------------------------------------
// Player constants

// units per second for stamina to regenerate with
const c_fStaminaRegenRate = 10.0;

// how many stamina points a jump consumes
const c_fStaminaCostForJump = 25.0;

// cost of stamina points per second while sprinting
const c_fStaminaSprintRate = 15.0;

// number of seconds to delay after stamina was consumed before starting auto-regen
const c_fStaminaRegenDelay = 2.0;


// ---------------------------------------------------------------------------------------------------------------

// randomly select a sound from a sound set
// a sound set is an array of sounds
// Example: [ "ANNETIRE4", "ANNETIRE1", "ANNETIRE2" ];
function RandSoundFromSet(sndset)
{
	return sndset[ TP_Engine.GetRandIntRanged(0, sndset.len()) ];
}


// ---------------------------------------------------------------------------------------------------------------
// custom player class supplying stamina functionality
//
// TScript:
//    string Script = "CPlayerTI"
//
class CPlayerTI extends CPlayer
{
	nvram = null;

	// sound set for stamina low verbalization, ANNETIRE4 is repeated a bunch time because the other sounds should have low probability
	sndsetStaminaLow = [ "ANNETIRE4","ANNETIRE4","ANNETIRE4","ANNETIRE4","ANNETIRE4","ANNETIRE4","ANNETIRE4", "ANNETIRE1","ANNETIRE2" ];

	/*********************************************************************************************
	---------------------------------------------------------
	NVRAM vars (vars that need to be preserved in savegames):
	---------------------------------------------------------
	
	// stamina value 0 - 100
	nvram.fStamina

	// a delay timer the prevents stamina regen (when it reaches 0 then regen is allowed)
	nvram.fStaminaRegenDelayTimer

	*********************************************************************************************/

	// true or false depending on if the sprint key is pressed or not
	bSprinting = null;


	function OnScriptInit(bSaveGame)
	{
		if (!bSaveGame)
		{
			// not loading a savegame, init default vals

			// create nvram table
			nvram = {};

			// create and init nvram vars
			nvram.fStamina <- 100.0
			nvram.fStaminaRegenDelayTimer <- 0.0;

			// make sure OnSimStep gets called
			ActivateSimStep(true);
		}

		// init non-saved vars
		bSprinting = false;
	}

	function OnSimStep(fStepDeltaSeconds)
	{
		// handle sprinting
		if (bSprinting)
		{
			ConsumeStamina(c_fStaminaSprintRate * fStepDeltaSeconds);
		}

		// update stamina regen delay timer
		if (nvram.fStaminaRegenDelayTimer > 0.0)
		{
			nvram.fStaminaRegenDelayTimer -= fStepDeltaSeconds;
			if (nvram.fStaminaRegenDelayTimer < 0.0)
			{
				// regen delay done, start regenerating now
				nvram.fStaminaRegenDelayTimer = 0.0;
				//Say("ANNETIRE5");
			}
		}

		// regenerate stamina
		if (nvram.fStamina < 100.0 && nvram.fStaminaRegenDelayTimer == 0.0)
		{
			nvram.fStamina += c_fStaminaRegenRate * fStepDeltaSeconds;
			if (nvram.fStamina > 100.0)
				nvram.fStamina = 100.0;
		}
	}

	// called when the player has been requested to jump, a return value of 1.0 results in standard jump height/force
	// (note that when this function gets called it doesn't mean that the player will definitely end up jumping)
	function fGetJumpFactor()
	{
		if (nvram.fStamina >= c_fStaminaCostForJump)
			// enough stamina for a jump
			return 1.0;
		else
			// no jump for you!
			return 0.0;
	}

	// called when setting player movement speed, fSpeedFactor is the unmodified speed factor
	// the speed factor is 0.5 for walk speed, around 0.78 for walk-strafe combo and 1.0 for run(/max) speed
	function fGetSpeedFactor(fSpeedFactor)
	{
		if (bSprinting && nvram.fStamina > 0.0)
		{
			// when sprinting always moves at maximum sprint speed, regardless of whether player is using the walk or run key
			return 1.75;
		}

		// reduce overall speed when out of stamina
		if (nvram.fStamina == 0.0)
		{
			return fSpeedFactor * 0.85;
		}

		return fSpeedFactor;
	}

	// called when jump is performed
	function OnJump()
	{
		ConsumeStamina(c_fStaminaCostForJump);
	}

	// call this whenever an action consumes stamina
	function ConsumeStamina(amount)
	{
		if (nvram.fStamina > 0.0)
		{
			local fPrevStamin = nvram.fStamina;

			nvram.fStamina -= amount;
			if (nvram.fStamina <= 0.0)
			{
				// just ran out of stamina
				nvram.fStamina = 0.0;
			}
			/*else if (nvram.fStamina < 80.0 && fPrevStamin >= 80.0)
			{
				// stamina just dropped below 80
				Say("ANNETIRE3");
			}*/
			/*else if (nvram.fStamina < 15.0 && fPrevStamin >= 15.0)
			{
				// stamina just dropped below 15
				Say( RandSoundFromSet(sndsetStaminaLow) );
			}*/
		}

		// each time stamina is consumed there's a delay before starting regen
		nvram.fStaminaRegenDelayTimer = c_fStaminaRegenDelay;
	}

	// called by game script during input handling to notify this script of the sprint key state
	function UpdateSprintState(bNewState)
	{
		if (bSprinting == bNewState)
			// nothing changed
			return;

		bSprinting = bNewState;
	}

	function GetStamina()
	{
		return nvram.fStamina;
	}
}
